from astropy.coordinates import CartesianRepresentation
from astropy.constants import G
from axinite.functions import vector_to, apply_to_vector, vector_magnitude, unit_vector
import astropy.units as u
from math import pi
from numpy import float64

class Body:
    def __init__(self, name, mass: u.Quantity, position: CartesianRepresentation, velocity: CartesianRepresentation, radius: u.Quantity, color:str="", light:bool=False, retain=None, radius_multiplier=1):
        self.mass = mass
        self.r = { float64(0): position}
        self.v = { float64(0): velocity}
        self.name = name
        self.radius = radius * radius_multiplier
        self.color = color
        self.light = light
        self.retain = retain
        self.radius_multiplier = radius_multiplier

    def gravitational_force(self, r: CartesianRepresentation, m: u.Quantity):
        return -G * ((self.mass * m) / vector_magnitude(r)**2) * unit_vector(r)


import axinite.tools as axtools
import matplotlib.pyplot as ply

def mpl_frontend(args: axtools.Body, mode: str):
    if mode != "show": raise Exception("mpl_frontend is only supported in show mode")



import axinite as ax
import axinite.tools as axtools
from vpython import *

def run(_args: axtools.AxiniteArgs, frontend):
    """Load and display a simulation simultaneously."""

    args = _args
    if args.rate is None:
        args.rate = 100
    if args.radius_multiplier is None:
        args.radius_multiplier = 1
    if args.retain is None:
       args.retain = 200

    args.action = frontend[0]
    try: ax.load(*args.unpack(), t=args.t)
    finally:
        frontend[1]()


import axinite as ax
from axinite.tools import AxiniteArgs, interpret_time, data_to_body
import astropy.units as u
import json

def read(path: str) -> AxiniteArgs:
    with open(path, 'r') as f:
        data = json.load(f)
        
        args = AxiniteArgs()
        args.name = data["name"]
        args.delta = interpret_time(data["delta"])
        args.limit = interpret_time(data["limit"])
        args.t = data["t"] * u.s

        if "radius_multiplier" in data:
            args.radius_multiplier = data["radius_multiplier"]

        if "rate" in data:
            args.rate = data["rate"]

        if "retain" in data:
            args.retain = data["retain"]

        if "frontend_args" in data:
            args.frontend_args = data["frontend_args"]

        for body in data["bodies"]: 
            args.bodies.append(data_to_body(body))

        return args


import axinite as ax

class AxiniteArgs:
    def __init__(self):
        self.name = None
        self.delta = None
        self.limit = None
        self.action = None
        self.t = None
        self.bodies: list = []
        self.radius_multiplier = None
        self.rate = None
        self.retain = None
        self.modifiers = []
        self.frontend_args = {}

    def unpack(self):
        return self.delta, self.limit, self.action, *self.bodies